@@ -1,4 +1,5 @@
 """The missing async toolbox"""
+
 from .builtins import (
     anext,
     zip,
@@ -18,13 +19,21 @@ from .builtins import (
     sorted,
 )
 from .functools import reduce, lru_cache, cache, cached_property
-from .contextlib import closing, contextmanager, nullcontext, ExitStack
+from .contextlib import (
+    closing,
+    ContextDecorator,
+    contextmanager,
+    nullcontext,
+    ExitStack,
+)
 from .itertools import (
     accumulate,
+    batched,
     cycle,
     chain,
     compress,
     dropwhile,
+    filterfalse,
     islice,
     takewhile,
     starmap,
@@ -36,7 +45,7 @@ from .itertools import (
 from .asynctools import borrow, scoped_iter, await_each, any_iter, apply, sync
 from .heapq import merge, nlargest, nsmallest
 
-__version__ = "3.10.5"
+__version__ = "3.12.4"
 
 __all__ = [
     "anext",
@@ -62,15 +71,18 @@ __all__ = [
     "cached_property",
     # contextlib
     "closing",
+    "ContextDecorator",
     "contextmanager",
     "nullcontext",
     "ExitStack",
     # itertools
     "accumulate",
+    "batched",
     "cycle",
     "chain",
     "compress",
     "dropwhile",
+    "filterfalse",
     "takewhile",
     "islice",
     "starmap",
