@@ -8,17 +8,16 @@ from typing import (
     Any,
     Awaitable,
     Deque,
-    overload,
+    AsyncContextManager,
 )
 from functools import wraps
 from collections import deque
 from functools import partial
 import sys
 
-from ._typing import Protocol, AsyncContextManager, ContextManager, T, C
+from ._typing import AClose, ContextManager, AC, T, C
 from ._core import awaitify
-from ._utility import public_module, slot_get as _slot_get
-
+from ._utility import public_module
 
 AnyContextManager = Union[AsyncContextManager[T], ContextManager[T]]
 
@@ -28,18 +27,10 @@ AnyContextManager = Union[AsyncContextManager[T], ContextManager[T]]
 AbstractContextManager = AsyncContextManager
 
 
-class ACloseable(Protocol):
-    async def aclose(self) -> None:
-        """Asynchronously close this object"""
-
-
-AC = TypeVar("AC", bound=ACloseable)
-
-
 def contextmanager(
     func: Callable[..., AsyncGenerator[T, None]]
 ) -> Callable[..., AsyncContextManager[T]]:
-    """
+    r"""
     Create an asynchronous context manager out of an asynchronous generator function
 
     This is intended as a decorator for an asynchronous generator function.
@@ -50,7 +41,7 @@ def contextmanager(
     .. code-block:: python3
 
         @contextmanager
-        async def Context(*args, **kwargs):
+        async def context(*args, **kwargs):
             # __aenter__
             yield  # context value
             # __aexit__
@@ -58,6 +49,9 @@ def contextmanager(
     Note that if an exception ends the context block, it gets re-raised at the ``yield``
     inside the asynchronous generator (via :py:meth:`~agen.athrow`). In order to handle
     this exception, the ``yield`` should be wrapped in a ``try`` statement.
+
+    The created context manager is a :py:class:`~.ContextDecorator` and can also be used
+    as a decorator. It is automatically entered when a decorated function is ``await``\ ed.
     """
 
     @wraps(func)
@@ -67,13 +61,62 @@ def contextmanager(
     return helper
 
 
-class _AsyncGeneratorContextManager(Generic[T]):
+class ContextDecorator(AsyncContextManager[T]):
+    """
+    Base class to turn an async context manager into a decorator as well
+
+    Inheriting from this class adds the scaffolding to automatically enter
+    an async context manager on awaiting any callable decorated with it:
+
+    .. code:: python3
+
+        class DecoratorAndContext(ContextDecorator):
+            async def __aenter__(self) -> Any:
+                print("entering", self)
+
+            async def __aexit__(self, *exc):
+                print("exiting", self)
+
+        @DecoratorAndContext()
+        async def func():
+            print("running some function...")
+
+    The context manager can still be used regularly in `async with` statements.
+
+    Since functions are decorated with an existing context manager instance,
+    the same instance is entered and exited on every call. If the context is
+    not safe to be entered multiple times or even concurrently it should implement
+    the method ``_recreate_cm(:Self) -> Self`` to create a copy of itself.
+    """
+
+    __slots__ = ()
+
+    def _recreate_cm(self):
+        """Return another instance of the context manager that is ready for entering again"""
+        # Default to assuming the CM is reentrant, concurrency safe, ... and just return itself.
+        # Since for the `async` case many CMs have to be this anyway, this should apply most.
+        return self
+
+    def __call__(self, func: AC, /) -> AC:
+        @wraps(func)
+        async def inner(*args: Any, **kwds: Any) -> Any:
+            async with self._recreate_cm():
+                return await func(*args, **kwds)
+
+        return inner  # type: ignore
+
+
+class _AsyncGeneratorContextManager(ContextDecorator[T]):
     def __init__(
         self, func: Callable[..., AsyncGenerator[T, None]], args: Any, kwds: Any
     ):
         self.gen = func(*args, **kwds)
+        self.__recreate_args = func, args, kwds
         self.__doc__ = getattr(func, "__doc__", type(self).__doc__)
 
+    def _recreate_cm(self):
+        return type(self)(*self.__recreate_args)
+
     async def __aenter__(self) -> T:
         try:
             return await self.gen.__anext__()
@@ -94,9 +137,9 @@ class _AsyncGeneratorContextManager(Generic[T]):
                 # Use `aclose` to have additional checks for the child to
                 # handle shutdown properly.
                 if exc_type is GeneratorExit:
-                    result = await self.gen.aclose()
+                    result = await self.gen.aclose()  # type: ignore
                 else:
-                    result = await self.gen.athrow(exc_type, exc_val, exc_tb)
+                    result = await self.gen.athrow(exc_val)
             except StopAsyncIteration as exc:
                 return exc is not exc_tb
             except RuntimeError as exc:
@@ -126,7 +169,7 @@ class _AsyncGeneratorContextManager(Generic[T]):
 
 
 @public_module(__name__, "closing")
-class Closing(Generic[AC]):
+class Closing(Generic[AClose]):
     """
     Create an :term:`asynchronous context manager` to ``aclose`` some ``thing`` on exit
 
@@ -150,22 +193,21 @@ class Closing(Generic[AC]):
                  is eventually closed and only :term:`borrowed <borrowing>` until then.
     """
 
-    def __init__(self, thing: AC):
+    def __init__(self, thing: AClose):
         self.thing = thing
 
-    async def __aenter__(self) -> AC:
+    async def __aenter__(self) -> AClose:
         return self.thing
 
-    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> bool:
+    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
         await self.thing.aclose()
-        return False
 
 
 closing = Closing
 
 
 @public_module(__name__, "nullcontext")
-class NullContext(Generic[T]):
+class NullContext(AsyncContextManager[T]):
     """
     Create an :term:`asynchronous context manager` that only returns ``enter_result``
 
@@ -190,44 +232,20 @@ class NullContext(Generic[T]):
 
     __slots__ = ("enter_result",)
 
-    @overload
-    def __init__(self: "NullContext[None]", enter_result: None = ...) -> None:
-        ...
-
-    @overload
-    def __init__(self: "NullContext[T]", enter_result: T) -> None:
-        ...
-
-    def __init__(self, enter_result: Optional[T] = None):
+    def __init__(self, enter_result: T = None):
         self.enter_result = enter_result
 
-    @overload
-    async def __aenter__(self: "NullContext[None]") -> None:
-        ...
-
-    @overload
-    async def __aenter__(self: "NullContext[T]") -> T:
-        ...
-
-    async def __aenter__(self) -> Optional[T]:
+    async def __aenter__(self) -> T:
         return self.enter_result
 
-    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> bool:
-        return False
+    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
+        return None
 
 
 nullcontext = NullContext
 
 
-SE = TypeVar(
-    "SE",
-    bound=Union[
-        AsyncContextManager[Any],
-        ContextManager[Any],
-        Callable[[Any, BaseException, Any], Optional[bool]],
-        Callable[[Any, BaseException, Any], Awaitable[Optional[bool]]],
-    ],
-)
+SE = TypeVar("SE")
 
 
 class ExitStack:
@@ -311,16 +329,18 @@ class ExitStack:
             instead.
         """
         try:
-            aexit = _slot_get(exit, "__aexit__")
+            aexit = exit.__aexit__  # type: ignore
         except AttributeError:
             try:
-                aexit = awaitify(_slot_get(exit, "__exit__"))
+                aexit = awaitify(
+                    exit.__exit__,  # type: ignore
+                )
             except AttributeError:
                 assert callable(
                     exit
                 ), f"Expected (async) context manager or callable, got {exit}"
                 aexit = awaitify(exit)
-        self._exit_callbacks.append(aexit)
+        self._exit_callbacks.append(aexit)  # pyright: ignore[reportUnknownArgumentType]
         return exit
 
     def callback(self, callback: C, *args: Any, **kwargs: Any) -> C:
@@ -334,7 +354,7 @@ class ExitStack:
         the exception handled by the stack. The callback is treated as
         :term:`async neutral`, i.e. it may be a synchronous function.
 
-        This method does not change its argument, and can be used as a context manager.
+        This method does not change its argument, and can be used as a decorator.
         """
         self._exit_callbacks.append(
             partial(self._aexit_callback, partial(awaitify(callback), *args, **kwargs))
@@ -371,14 +391,16 @@ class ExitStack:
         either.
         """
         try:
-            aexit = _slot_get(cm, "__aexit__")
+            aexit = cm.__aexit__  # type: ignore
         except AttributeError:
-            aexit = awaitify(_slot_get(cm, "__exit__"))
-            context_value = _slot_get(cm, "__enter__")()
+            aexit = awaitify(
+                cm.__exit__,  # type: ignore
+            )
+            context_value = cm.__enter__()  # type: ignore
         else:
-            context_value = await _slot_get(cm, "__aenter__")()
-        self._exit_callbacks.append(aexit)
-        return context_value  # type: ignore
+            context_value = await cm.__aenter__()  # type: ignore
+        self._exit_callbacks.append(aexit)  # pyright: ignore[reportUnknownArgumentType]
+        return context_value  # pyright: ignore[reportUnknownVariableType]
 
     async def aclose(self) -> None:
         """
@@ -434,12 +456,12 @@ class ExitStack:
                     suppress_exc = True
                     reraise_exc = False
                     exc_type = exc_val = tb = None
-            except BaseException as exc:
+            except BaseException as exc:  # noqa: B036
                 # simulate the stack of exceptions by setting the context
                 self._stitch_context(exc, exc_val, unwind_context)
                 reraise_exc = True
                 exc_type, exc_val, tb = type(exc), exc, exc.__traceback__
-        if reraise_exc:
+        if reraise_exc and exc_val is not None:
             # The __context__ is replaced by a normal `raise`, and only
             # preserved by a bare `raise` in an except block.
             exc_context = exc_val.__context__
