@@ -1,6 +1,7 @@
 from typing import (
     Any,
     TypeVar,
+    AsyncContextManager,
     AsyncIterator,
     List,
     Awaitable,
@@ -17,7 +18,7 @@ from typing import (
 )
 from collections import deque
 
-from ._typing import T, R, T1, T2, T3, T4, T5, AnyIterable, ADD, AsyncContextManager
+from ._typing import ACloseable, T, AnyIterable, ADD
 from ._utility import public_module
 from ._core import (
     ScopedIter,
@@ -25,7 +26,13 @@ from ._core import (
     Sentinel,
     borrow as _borrow,
 )
-from .builtins import anext, zip, enumerate as aenumerate, iter as aiter
+from .builtins import (
+    anext,
+    zip,
+    enumerate as aenumerate,
+    iter as aiter,
+    tuple as atuple,
+)
 
 S = TypeVar("S")
 
@@ -46,7 +53,7 @@ async def cycle(iterable: AnyIterable[T]) -> AsyncIterator[T]:
     """
     buffer: List[T] = []
     async with ScopedIter(iterable) as async_iter:
-        async for item in async_iter:  # type: T
+        async for item in async_iter:
             buffer.append(item)
             yield item
     if not buffer:
@@ -64,34 +71,6 @@ async def add(x: ADD, y: ADD) -> ADD:
     return x + y
 
 
-@overload
-def accumulate(iterable: AnyIterable[ADD]) -> AsyncIterator[ADD]:
-    ...
-
-
-@overload
-def accumulate(iterable: AnyIterable[ADD], *, initial: ADD) -> AsyncIterator[ADD]:
-    ...
-
-
-@overload
-def accumulate(
-    iterable: AnyIterable[T],
-    function: Union[Callable[[T, T], T], Callable[[T, T], Awaitable[T]]],
-) -> AsyncIterator[T]:
-    ...
-
-
-@overload
-def accumulate(
-    iterable: AnyIterable[T],
-    function: Union[Callable[[T, T], T], Callable[[T, T], Awaitable[T]]],
-    *,
-    initial: T,
-) -> AsyncIterator[T]:
-    ...
-
-
 async def accumulate(
     iterable: AnyIterable[Any],
     function: Union[
@@ -141,39 +120,77 @@ async def accumulate(
             yield value
 
 
-async def chain(*iterables: AnyIterable[T]) -> AsyncIterator[T]:
+async def batched(iterable: AnyIterable[T], n: int) -> AsyncIterator[Tuple[T, ...]]:
+    """
+    Batch the ``iterable`` to tuples of the length ``n``.
+
+    This lazily exhausts ``iterable`` and returns each batch as soon as it's ready.
+    """
+    if n < 1:
+        raise ValueError("n must be at least one")
+    async with ScopedIter(iterable) as item_iter:
+        while batch := await atuple(islice(_borrow(item_iter), n)):
+            yield batch
+
+
+class chain(AsyncIterator[T]):
     """
     An :term:`asynchronous iterator` flattening values from all ``iterables``
 
     The resulting iterator consecutively iterates over and yields all values from
     each of the ``iterables``. This is similar to converting all ``iterables`` to
     sequences and concatenating them, but lazily exhausts each iterable.
+
+    The ``chain`` assumes ownership of its ``iterables`` and closes them reliably
+    when the ``chain`` is closed. Pass the ``iterables`` via a :py:class:`tuple` to
+    ``chain.from_iterable`` to avoid closing all iterables but those already processed.
     """
-    for iterable in iterables:
-        async with ScopedIter(iterable) as iterator:
-            async for item in iterator:
-                yield item
 
+    __slots__ = ("_iterator", "_owned_iterators")
 
-@public_module(__name__, "chain.from_iterable")
-async def chain_from_iterable(
-    iterable: AnyIterable[AnyIterable[T]],
-) -> AsyncIterator[T]:
-    """
-    Alternate constructor for :py:func:`~.chain` that lazily exhausts iterables as well
-    """
-    async with ScopedIter(iterable) as iterables:
-        async for sub_iterable in iterables:
-            async with ScopedIter(sub_iterable) as iterator:
-                async for item in iterator:
-                    yield item
+    @staticmethod
+    async def _chain_iterator(
+        any_iterables: AnyIterable[AnyIterable[T]],
+    ) -> AsyncGenerator[T, None]:
+        async with ScopedIter(any_iterables) as iterables:
+            async for iterable in iterables:
+                async with ScopedIter(iterable) as iterator:
+                    async for item in iterator:
+                        yield item
 
+    def __init__(
+        self, *iterables: AnyIterable[T], _iterables: AnyIterable[AnyIterable[T]] = ()
+    ):
+        self._iterator = self._chain_iterator(iterables or _iterables)
+        self._owned_iterators = tuple(
+            iterable  # type: ignore[misc]
+            for iterable in iterables
+            if isinstance(iterable, AsyncIterator) and isinstance(iterable, ACloseable)
+        )
 
-chain.from_iterable = chain_from_iterable  # type: ignore
+    @classmethod
+    def from_iterable(cls, iterable: AnyIterable[AnyIterable[T]]) -> "chain[T]":
+        """
+        Alternate constructor for :py:func:`~.chain` that lazily exhausts
+        the ``iterable`` of iterables as well
+
+        This is suitable for chaining iterables from a lazy or infinite ``iterable``.
+        In turn, closing the ``chain`` only closes those iterables
+        already fetched from ``iterable``.
+        """
+        return cls(_iterables=iterable)
+
+    def __anext__(self) -> Awaitable[T]:
+        return self._iterator.__anext__()
+
+    async def aclose(self) -> None:
+        for iterable in self._owned_iterators:
+            await iterable.aclose()
+        await self._iterator.aclose()
 
 
 async def compress(
-    data: AnyIterable[T], selectors: AnyIterable[bool]
+    data: AnyIterable[T], selectors: AnyIterable[Any]
 ) -> AsyncIterator[T]:
     """
     An :term:`asynchronous iterator` for items of ``data`` with true ``selectors``
@@ -194,8 +211,7 @@ async def compress(
 
 
 async def dropwhile(
-    predicate: Union[Callable[[T], bool], Callable[[T], Awaitable[bool]]],
-    iterable: AnyIterable[T],
+    predicate: Callable[[T], Any], iterable: AnyIterable[T]
 ) -> AsyncIterator[T]:
     """
     Yield items from ``iterable`` after ``predicate(item)`` is no longer true
@@ -216,6 +232,24 @@ async def dropwhile(
             yield item
 
 
+async def filterfalse(
+    predicate: Union[Callable[[T], bool], Callable[[T], Awaitable[bool]], None],
+    iterable: AnyIterable[T],
+) -> AsyncIterator[T]:
+    """
+    Yield items from ``iterable`` for which ``predicate(item)`` is false.
+
+    If ``predicate`` is ``None``, yield any items which are false.
+    """
+    async with ScopedIter(iterable) as async_iter:
+        if predicate is None:
+            predicate = bool
+        predicate = _awaitify(predicate)
+        async for item in async_iter:
+            if not await predicate(item):
+                yield item
+
+
 async def islice(iterable: AnyIterable[T], *args: Optional[int]) -> AsyncIterator[T]:
     """
     An :term:`asynchronous iterator` over items from ``iterable`` in a :py:class:`slice`
@@ -273,8 +307,7 @@ async def starmap(
 
 
 async def takewhile(
-    predicate: Union[Callable[[T], bool], Callable[[T], Awaitable[bool]]],
-    iterable: AnyIterable[T],
+    predicate: Callable[[T], Any], iterable: AnyIterable[T]
 ) -> AsyncIterator[T]:
     """
     Yield items from ``iterable`` as long as ``predicate(item)`` is true
@@ -297,11 +330,13 @@ async def takewhile(
 class NoLock:
     """Dummy lock that provides the proper interface but no protection"""
 
+    __slots__ = ()
+
     async def __aenter__(self) -> None:
         pass
 
-    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> bool:
-        return False
+    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
+        return None
 
 
 async def tee_peer(
@@ -341,8 +376,8 @@ async def tee_peer(
                 peers.pop(idx)
                 break
         # if we are the last peer, try and close the iterator
-        if not peers and hasattr(iterator, "aclose"):
-            await iterator.aclose()  # type: ignore
+        if not peers and isinstance(iterator, ACloseable):
+            await iterator.aclose()
 
 
 @public_module(__name__, "tee")
@@ -384,6 +419,8 @@ class Tee(Generic[T]):
     and access is automatically synchronised.
     """
 
+    __slots__ = ("_iterator", "_buffers", "_children")
+
     def __init__(
         self,
         iterable: AnyIterable[T],
@@ -407,12 +444,10 @@ class Tee(Generic[T]):
         return len(self._children)
 
     @overload
-    def __getitem__(self, item: int) -> AsyncIterator[T]:
-        ...
+    def __getitem__(self, item: int) -> AsyncIterator[T]: ...
 
     @overload
-    def __getitem__(self, item: slice) -> Tuple[AsyncIterator[T], ...]:
-        ...
+    def __getitem__(self, item: slice) -> Tuple[AsyncIterator[T], ...]: ...
 
     def __getitem__(
         self, item: Union[int, slice]
@@ -425,9 +460,8 @@ class Tee(Generic[T]):
     async def __aenter__(self) -> "Tee[T]":
         return self
 
-    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> bool:
+    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
         await self.aclose()
-        return False
 
     async def aclose(self) -> None:
         for child in self._children:
@@ -458,76 +492,6 @@ async def _repeat(value: T) -> AsyncIterator[T]:
         yield value
 
 
-@overload
-def zip_longest(
-    __it1: AnyIterable[T1],
-    *,
-    fillvalue: S = ...,
-) -> AsyncIterator[Tuple[T1]]:
-    ...
-
-
-@overload
-def zip_longest(
-    __it1: AnyIterable[T1],
-    __it2: AnyIterable[T2],
-    *,
-    fillvalue: S = ...,
-) -> AsyncIterator[Tuple[Union[T1, S], Union[T2, S]]]:
-    ...
-
-
-@overload
-def zip_longest(
-    __it1: AnyIterable[T1],
-    __it2: AnyIterable[T2],
-    __it3: AnyIterable[T3],
-    *,
-    fillvalue: S = ...,
-) -> AsyncIterator[Tuple[Union[T1, S], Union[T2, S], Union[T3, S]]]:
-    ...
-
-
-@overload
-def zip_longest(
-    __it1: AnyIterable[T1],
-    __it2: AnyIterable[T2],
-    __it3: AnyIterable[T3],
-    __it4: AnyIterable[T4],
-    *,
-    fillvalue: S = ...,
-) -> AsyncIterator[Tuple[Union[T1, S], Union[T2, S], Union[T3, S], Union[T4, S]]]:
-    ...
-
-
-@overload
-def zip_longest(
-    __it1: AnyIterable[T1],
-    __it2: AnyIterable[T2],
-    __it3: AnyIterable[T3],
-    __it4: AnyIterable[T4],
-    __it5: AnyIterable[T5],
-    *,
-    fillvalue: S = ...,
-) -> AsyncIterator[
-    Tuple[Union[T1, S], Union[T2, S], Union[T3, S], Union[T4, S], Union[T5, S]]
-]:
-    ...
-
-
-@overload
-def zip_longest(
-    __it1: AnyIterable[Any],
-    __it2: AnyIterable[Any],
-    __it3: AnyIterable[Any],
-    __it4: AnyIterable[Any],
-    __it5: AnyIterable[Any],
-    *iterables: AnyIterable[Any],
-    fillvalue: S = ...,
-) -> AsyncIterator[Tuple[Any, ...]]:
-    ...
-
-
 async def zip_longest(
     *iterables: AnyIterable[Any], fillvalue: Any = None
 ) -> AsyncIterator[Tuple[Any, ...]]:
@@ -552,7 +516,7 @@ async def zip_longest(
     try:
         remaining = len(async_iters)
         while True:
-            values = []
+            values: list[Any] = []
             for index, aiterator in enumerate(async_iters):
                 try:
                     value = await anext(aiterator)
@@ -569,12 +533,8 @@ async def zip_longest(
     finally:
         await fill_iter.aclose()  # type: ignore
         for iterator in async_iters:
-            try:
-                aclose = iterator.aclose()  # type: ignore
-            except AttributeError:
-                pass
-            else:
-                await aclose
+            if isinstance(iterator, ACloseable):
+                await iterator.aclose()
 
 
 async def identity(x: T) -> T:
@@ -582,28 +542,7 @@ async def identity(x: T) -> T:
     return x
 
 
-@overload  # noqa: F811
-def groupby(  # noqa: F811
-    iterable: AnyIterable[T],
-) -> AsyncIterator[Tuple[T, AsyncIterator[T]]]:
-    ...
-
-
-@overload  # noqa: F811
-def groupby(  # noqa: F811
-    iterable: AnyIterable[T], key: None
-) -> AsyncIterator[Tuple[T, AsyncIterator[T]]]:
-    ...
-
-
-@overload  # noqa: F811
-def groupby(  # noqa: F811
-    iterable: AnyIterable[T], key: Union[Callable[[T], R], Callable[[T], Awaitable[R]]]
-) -> AsyncIterator[Tuple[R, AsyncIterator[T]]]:
-    ...
-
-
-async def groupby(  # noqa: F811
+async def groupby(
     iterable: AnyIterable[Any],
     key: Optional[
         Union[Callable[[Any], Any], Callable[[Any], Awaitable[Any]]]
@@ -631,7 +570,7 @@ async def groupby(  # noqa: F811
     # whether the current group was exhausted and the next begins already
     exhausted = False
     # `current_*`: buffer for key/value the current group peeked beyond its end
-    current_key = current_value = nothing = object()  # type: Any
+    current_key = current_value = nothing = object()
     make_key: Callable[[Any], Awaitable[Any]] = (
         _awaitify(key) if key is not None else identity  # type: ignore
     )
