@@ -42,7 +42,7 @@ async def inside_loop():
     return await signal is signal
 
 
-def sync(test_case: Callable[..., Coroutine]):
+def sync(test_case: Callable[..., Coroutine[T, Any, Any]]) -> Callable[..., T]:
     """
     Mark an ``async def`` test case to be run synchronously
 
@@ -51,7 +51,7 @@ def sync(test_case: Callable[..., Coroutine]):
     """
 
     @wraps(test_case)
-    def run_sync(*args, **kwargs):
+    def run_sync(*args: Any, **kwargs: Any) -> T:
         coro = test_case(*args, **kwargs)
         try:
             event = None
@@ -71,7 +71,7 @@ def sync(test_case: Callable[..., Coroutine]):
 class Schedule:
     """Signal to the event loop to adopt and run a new coroutine"""
 
-    def __init__(self, *coros: Coroutine):
+    def __init__(self, *coros: Coroutine[Any, Any, Any]):
         self.coros = coros
 
     def __await__(self):
@@ -85,17 +85,39 @@ class Switch:
         yield self
 
 
-def multi_sync(test_case: Callable[..., Coroutine]):
+class Lock:
+    def __init__(self):
+        self._owned = False
+        self._waiting: list[object] = []
+
+    async def __aenter__(self):
+        if self._owned:
+            # wait until it is our turn to take the lock
+            token = object()
+            self._waiting.append(token)
+            while self._owned or self._waiting[0] is not token:
+                await Switch()
+            # take the lock and remove our wait claim
+            self._owned = True
+            self._waiting.pop(0)
+        self._owned = True
+
+    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any):
+        self._owned = False
+
+
+def multi_sync(test_case: Callable[..., Coroutine[T, Any, Any]]) -> Callable[..., T]:
     """
-    Mark an ``async def`` test case to be run synchronously with chicldren
+    Mark an ``async def`` test case to be run synchronously with children
 
     This emulates a primitive "event loop" which only responds
-    to the :py:class:`PingPong`, :py:class:`Schedule` and :py:class:`Switch`.
+    to the :py:class:`PingPong`, :py:class:`Schedule`, :py:class:`Switch`
+    and :py:class:`Lock`.
     """
 
     @wraps(test_case)
-    def run_sync(*args, **kwargs):
-        run_queue: Deque[Tuple[Coroutine, Any]] = deque()
+    def run_sync(*args: Any, **kwargs: Any):
+        run_queue: Deque[Tuple[Coroutine[Any, Any, Any], Any]] = deque()
         run_queue.append((test_case(*args, **kwargs), None))
         while run_queue:
             coro, event = run_queue.popleft()
@@ -103,7 +125,7 @@ def multi_sync(test_case: Callable[..., Coroutine]):
                 event = coro.send(event)
             except StopIteration as e:
                 result = e.args[0] if e.args else None
-                assert result is None
+                assert result is None, f"got '{result!r}' expected 'None'"
             else:
                 if isinstance(event, PingPong):
                     run_queue.appendleft((coro, event))
